@@ -804,4 +804,13 @@ class InlineTest extends TestCase
     {
         $this->assertSame(array('foo' => 'bar'), Inline::parse('{ foo: !str bar }'));
     }
+
+    /**
+     * @expectedException \Symfony\Component\Yaml\Exception\ParseException
+     * @expectedExceptionMessage Unexpected end of line, expected one of ",}" at line 1 (near "{abc: 'def'").
+     */
+    public function testUnfinishedInlineMap()
+    {
+        Inline::parse("{abc: 'def'");
+    }
 }
